--[[
--
-- Ideas
--  (which means they are *ideas* - NOT necessarily things I *want* to be
--  implemented)
--
--  - Some sort of text progress bar like [#####       ] (or just force ckk to
--  implement :AddFrame)
--  - Fix coloring.
--  - Try to flash or increase font size of individual lines as they near
--  completion.
--  - Ability to collapse/expand a module like QuestsFu does with zones in its
--  tablet.
--  - Some sort of magic when you click a timer. For example use it to
--  increase/decrease font size of that line or freeze it in place so that it's
--  not removed when the timer reaches 0 or whatever.
--  - Set background color according to severity
--  - Set up a hash-based coloring system for the module names, so that we're
--  almost guaranteed to have all the modules listed in different colors but
--  they should have the same color every time. I believe Firefox has a few tab
--  addons that do this, and one is called ChromaTabs.
--]]


local name = "BigWigs Tablet Bars"
local L = AceLibrary("AceLocale-2.2"):new(name)
local tablet = AceLibrary("Tablet-2.0")
local bars = nil

L:RegisterTranslations("enUS", function() return {
	["tabletbars"] = true,
	["Tablet Bars"] = true,
	["Alternative bar display for Big Wigs."] = true,
} end )

BWTabletBars = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
BWTabletBars:RegisterDB("BigWigsTimerBarsDB")
BWTabletBars:RegisterDefaults("profile", {
	wtf = "lol",
})

BWTabletBars.hideWithoutStandby = true
BWTabletBars.hasIcon = "Interface\\Icons\\INV_Gizmo_01"
BWTabletBars.hasNoColor = true
BWTabletBars.hasNoText = true
BWTabletBars.defaultMinimapPosition = 60
BWTabletBars.clickableTooltip = true
BWTabletBars.defaultPosition = "RIGHT"
--BWTabletBars.tooltipHiddenWhenEmpty = true

-------------------------------------------------------------------------------
-- Event and initialization                                                  --
-------------------------------------------------------------------------------

function BWTabletBars:OnEnable()
	self:RegisterEvent("BigWigs_StartBar")
	self:RegisterEvent("BigWigs_StopBar")

	self:RegisterEvent("AceEvent_FullyInitialized")
end

function BWTabletBars:AceEvent_FullyInitialized()
	DEFAULT_CHAT_FRAME:AddMessage("|cffff0000BigWigs Tablet Bars is a proof-of-concept addon and it is not meant for use. If anyone wants to make it better and usable, be my guest.|r")
end

function BWTabletBars:BigWigs_StartBar(module, text, time, icon)
	if not text or not time then return end
	local key = module.name or module:ToString()
	if not bars then bars = {} end
	if not bars[key] then bars[key] = {} end
	bars[key][text] = {
		["time"] = time,
		["icon"] = icon,
	}
	self:ScheduleRepeatingEvent(key..text, function()
		bars[key][text].time = bars[key][text].time - 1
		if bars[key][text].time <= 0 then
			self:CancelScheduledEvent(key..text)
			bars[key][text] = nil
		end
	end, 1, self)
	self:HandleUpdates()
end

function BWTabletBars:BigWigs_StopBar(module, text)
	if bars[module.name] then bars[module.name][text] = nil end
	self:HandleUpdates()
end

-------------------------------------------------------------------------------
-- Tablet handling                                                           --
-------------------------------------------------------------------------------

function BWTabletBars:HandleUpdates()
	if bars then
		self:ScheduleRepeatingEvent("displayUpdate", self.UpdateDisplay, 1, self)
	else
		self:ScheduleRepeatingEvent("displayUpdate", self.UpdateDisplay, 60, self)
	end
end

function BWTabletBars:OnTooltipUpdate()
	if not bars then return end

	local moduleCategories = {}
	for k, v in pairs(bars) do
		if not moduleCategories[k] then
			table.insert(moduleCategories, k)
		end
	end
	table.sort(moduleCategories)
	for i, category in ipairs(moduleCategories) do
		local cat = tablet:AddCategory(
			"columns", 2,
			"text", category,
			"child_justify", "LEFT",
			"child_justify2", "RIGHT",
			"child_wrap", false,
			"child_wrap2", false,
			"child_hasCheck", true,
			"child_checked", true,
			"showWithoutChildren", false
		)
		for k, v in pairs(bars[category]) do
			if v.time >= 0 then
				cat:AddLine(
					"text", self:ColorText(k, v.time),
					"text2", self:ColorText(v.time, v.time),
					"checkIcon", v.icon
				)	
			end
		end
	end
end

function BWTabletBars:ColorText(input, time)
	if BigWigsColors and type(BigWigsColors) == "table" and type(BigWigsColors.BarColor) == "function" then
		return "|cff"..BigWigsColors:BarColor(time)..input.."|r"
	else
		return input
	end
end


